#include <iostream>
#include <utility>
#include <vector>

using namespace std;

const int MAXN = 1e5 + 3;

vector<int> graph[MAXN];
int component[MAXN], parent[MAXN], depth[MAXN];

void dfs(int currv, int component_id) {
    if (parent[currv] == -1) {
        depth[currv] = 1;
    } else {
        depth[currv] = depth[parent[currv]] + 1;
    }

    component[currv] = component_id;

    for (int nextv : graph[currv]) {
        if (nextv == parent[currv]) {
            continue;
        }

        parent[nextv] = currv;
        dfs(nextv, component_id);
    }
}

void initialize(int ID, int N, int M, int Q, vector<pair<int,int>> friendships) {
    for (auto [x, y] : friendships) {
        --x;
        --y;

        graph[x].push_back(y);
        graph[y].push_back(x);
    }

    for (int i = 0; i < N; ++i) {
        component[i] = parent[i] = -1;
        depth[i] = 0;
    }

    int component_id = 1;
    for (int i = 0; i < N; ++i) {
        if (component[i] == -1) {
            dfs(i, component_id);
            ++component_id;
        }
    }
}

void connect(int ID, int x, int y) {
    --x;
    --y;

    graph[x].push_back(y);
    graph[y].push_back(x);
}

bool question(int ID, int x, int y) {
    --x;
    --y;

    if (component[x] != component[y]) {
        return false;
    }

    if (parent[x] == parent[y]) {
        return true;
    }

    if (depth[x] < depth[y]) {
        swap(x, y);
    }

    return (depth[y] + 2 == depth[x]) and (parent[parent[x]] == y);
}


